@@ -43,21 +43,22 @@ if sys.version_info < (3, 7):
 
 def _maybe_disable_warnings():
     import os
-    true = "IS_TRUE"
-    if not os.environ.get("ZFIT_DISABLE_TF_WARNINGS", true):
-        return
-    elif true:
-        warnings.warn("All TensorFlow warnings are by default suppressed by zfit."
-                      " In order to not suppress them,"
-                      " set the environment variable ZFIT_DISABLE_TF_WARNINGS to 0."
+    disable_warnings = os.environ.get("ZFIT_DISABLE_TF_WARNINGS")
+    if disable_warnings is None:
+        warnings.warn("TensorFlow warnings are by default suppressed by zfit."
+                      " In order to show them,"
+                      " set the environment variable ZFIT_DISABLE_TF_WARNINGS=0."
                       " In order to suppress the TensorFlow warnings AND this warning,"
-                      " set ZFIT_DISABLE_TF_WARNINGS manually to 1.")
+                      " set ZFIT_DISABLE_TF_WARNINGS=1.")
+    elif disable_warnings == '0':
+        return
+
     os.environ["KMP_AFFINITY"] = "noverbose"
     os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2'
 
     import tensorflow as tf
 
-    tf.compat.v1.logging.set_verbosity(tf.compat.v1.logging.ERROR)
+    tf.get_logger().setLevel('ERROR')
 
 
 _maybe_disable_warnings()
