@@ -1,11 +1,10 @@
-"""
-Functors are functions that take typically one or more other PDF. Prominent examples are a sum, convolution etc.
+"""Functors are functions that take typically one or more other PDF. Prominent examples are a sum, convolution etc.
 
 A FunctorBase class is provided to make handling the models easier.
 
 Their implementation is often non-trivial.
 """
-#  Copyright (c) 2020 zfit
+#  Copyright (c) 2021 zfit
 import functools
 import operator
 from collections import OrderedDict
@@ -17,15 +16,17 @@ import tensorflow as tf
 from .. import z
 from ..core.basepdf import BasePDF
 from ..core.coordinates import convert_to_obs_str
-from ..core.interfaces import ZfitPDF, ZfitModel, ZfitData
+from ..core.interfaces import ZfitData, ZfitModel, ZfitPDF
 from ..core.parameter import convert_to_parameter
 from ..core.space import supports
 from ..models.basefunctor import FunctorMixin, extract_daughter_input_obs
-from ..settings import ztypes, run
+from ..settings import run, ztypes
 from ..util import ztyping
 from ..util.container import convert_to_container
-from ..util.exception import (ModelIncompatibleError, ObsIncompatibleError, NormRangeUnderdefinedError,
-                              AnalyticIntegralNotImplementedError, SpecificFunctionNotImplementedError)
+from ..util.exception import (AnalyticIntegralNotImplemented,
+                              ModelIncompatibleError,
+                              NormRangeUnderdefinedError, ObsIncompatibleError,
+                              SpecificFunctionNotImplemented)
 from ..util.warnings import warn_advanced_feature, warn_changed_feature
 from ..z.random import counts_multinomial
 
@@ -128,7 +129,7 @@ class SumPDF(BaseFunctor):
             if len(fracs) == len(pdfs) - 1:
                 remaining_frac_func = lambda: tf.constant(1., dtype=ztypes.float) - tf.add_n(fracs)
                 remaining_frac = convert_to_parameter(remaining_frac_func,
-                                                      dependents=fracs)
+                                                      params=fracs)
                 if run.numeric_checks:
                     tf.debugging.assert_non_negative(remaining_frac,
                                                      f"The remaining fraction is negative, the sum of fracs is > 0. Fracs: {fracs}")  # check fractions
@@ -156,9 +157,9 @@ class SumPDF(BaseFunctor):
                 return tf.reduce_sum(
                     input_tensor=[tf.convert_to_tensor(value=y, dtype_hint=ztypes.float) for y in yields])
 
-            sum_yields = convert_to_parameter(sum_yields_func, dependents=yields)
+            sum_yields = convert_to_parameter(sum_yields_func, params=yields)
             yield_fracs = [convert_to_parameter(lambda sum_yields, yield_: yield_ / sum_yields,
-                                                dependents=[sum_yields, yield_])
+                                                params=[sum_yields, yield_])
                            for yield_ in yields]
 
             fracs_cleaned = None
@@ -170,7 +171,7 @@ class SumPDF(BaseFunctor):
 
         params = OrderedDict()
         for i, frac in enumerate(param_fracs):
-            params['frac_{}'.format(i)] = frac
+            params[f'frac_{i}'] = frac
 
         super().__init__(pdfs=pdfs, obs=obs, params=params, name=name)
         if all_extended and not fracs_cleaned:
@@ -198,7 +199,7 @@ class SumPDF(BaseFunctor):
     def _pdf(self, x, norm_range):  # NOT _pdf, as the normalization range can differ
         equal_norm_ranges = len(set([pdf.norm_range for pdf in self.pdfs] + [norm_range])) == 1
         if not equal_norm_ranges:
-            raise SpecificFunctionNotImplementedError
+            raise SpecificFunctionNotImplemented
         pdfs = self.pdfs
         fracs = self.params.values()
         probs = [pdf.pdf(x) * frac for pdf, frac in zip(pdfs, fracs)]
@@ -227,8 +228,8 @@ class SumPDF(BaseFunctor):
         try:
             integrals = [frac * pdf.analytic_integrate(limits=limits)  # do NOT propagate the norm_range!
                          for pdf, frac in zip(pdfs, fracs)]
-        except AnalyticIntegralNotImplementedError as error:
-            raise AnalyticIntegralNotImplementedError(
+        except AnalyticIntegralNotImplemented as error:
+            raise AnalyticIntegralNotImplemented(
                 f"analytic_integrate of pdf {self.name} is not implemented in this"
                 f" SumPDF, as at least one sub-pdf does not implement it.") from error
 
@@ -258,8 +259,8 @@ class SumPDF(BaseFunctor):
             partial_integral = [pdf.partial_analytic_integrate(x=x, limits=limits) * frac
                                 # do NOT propagate the norm_range!
                                 for pdf, frac in zip(pdfs, fracs)]
-        except AnalyticIntegralNotImplementedError as error:
-            raise AnalyticIntegralNotImplementedError(
+        except AnalyticIntegralNotImplemented as error:
+            raise AnalyticIntegralNotImplemented(
                 "partial_analytic_integrate of pdf {name} is not implemented in this"
                 " SumPDF, as at least one sub-pdf does not implement it.") from error
         partial_integral = functools.reduce(operator.add, partial_integral)
@@ -280,6 +281,7 @@ class SumPDF(BaseFunctor):
                 sub_sample = sub_sample.value()
             samples.append(sub_sample)
         sample = tf.concat(samples, axis=0)
+        sample = tf.random.shuffle(sample)
         return sample
 
 
@@ -301,4 +303,4 @@ class ProductPDF(BaseFunctor):  # TODO: compose of smaller Product PDF by disass
             return z.convert_to_tensor(functools.reduce(operator.mul, probs))
             # return tf.reduce_prod(input_tensor=probs, axis=0)
         else:
-            raise SpecificFunctionNotImplementedError
+            raise SpecificFunctionNotImplemented
